# -*- coding: utf-8 -*-
from __future__ import absolute_import

"""DWho Redis Adapter"""

__author__  = "Adrien DELLE CAVE <adc@doowan.net>"
__license__ = """
    Copyright (C) 2016  doowan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA..
"""

import redis


class DWhoAdapterRedis(object):
    def __init__(self, config, prefix = None, load = True):
        self.config  = config
        self.servers = {}

        if load:
            self.load(prefix)

    def load(self, prefix = None):
        for name in self.config['general']['redis'].iterkeys():
            if not prefix or name.startswith(prefix):
                self.connect(name)

    def connect(self, name):
        if not self.servers:
            self.servers = {name:
                            {'conn':    None,
                             'options': {}}}

        if not self.servers[name]['conn']:
            for key, value in self.config['general']['redis'][name].iteritems():
                if key == 'url':
                    self.servers[name]['conn'] = redis.Redis().from_url(key)
                self.servers[name]['options'][key] = value

        return self.servers[name]

    def set_key(self, key, val, expire = None, servers = None, prefix = None):
        if not servers:
            servers = self.servers

        if expire is not None:
            for name, server in servers.iteritems():
                if not prefix or name.startswith(prefix):
                    server['conn'].setex(key, val, expire)
        else:
            for name, server in servers.iteritems():
                if not prefix or name.startswith(prefix):
                    server['conn'].set(key, val)

    def get_key(self, key, servers = None, prefix = None):
        if not servers:
            servers = self.servers

        for name, server in servers.iteritems():
            if (not prefix or name.startswith(prefix)) \
               and server['conn'].exists(key):
                return server['conn'].get(key)

    def disconnect(self, name):
        if not self.servers:
            self.servers = {name:
                            {'conn':     None,
                             'options':  {}}}

        if self.servers[name]['conn']:
            try:
                self.servers[name]['conn'].disconnect()
            except Exception:
                pass

        self.servers[name]['conn']   = None

        return self
