# -*- coding: utf-8 -*-
"""dwho configuration"""

__author__  = "Adrien DELLE CAVE <adc@doowan.net>"
__license__ = """
    Copyright (C) 2015  doowan

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA..
"""

import logging
import signal
import yaml

from dwho.classes.modules import MODULES
from dwho.classes.plugins import PLUGINS
from socket import getfqdn

LOG             = logging.getLogger('dwho.config')

MAX_BODY_SIZE   = 8388608
MAX_WORKERS     = 1
SUBDIR_LEVELS   = 0
SUBDIR_CHARS    = "abcdef0123456789"

DWHO_THREADS    = []


def stop(signum, stack_frame):
    for thread in DWHO_THREADS:
        thread()

def parse_conf(conf):
    if not conf['general'].get('server_id'):
        conf['general']['server_id'] = getfqdn()

    if not conf['general'].get('max_body_size'):
        conf['general']['max_body_size'] = MAX_BODY_SIZE

    if not conf['general'].get('max_workers'):
        conf['general']['max_workers'] = MAX_WORKERS

    if not conf['general'].has_key('auth_basic_file'):
        conf['general']['auth_basic'] = None
        conf['general']['auth_basic_file'] = None

    if not conf['general'].has_key('subdir_levels'):
        conf['general']['subdir_levels'] = SUBDIR_LEVELS
    conf['general']['subdir_levels'] = int(conf['general']['subdir_levels'])

    if not conf['general'].has_key('subdir_chars'):
        conf['general']['subdir_chars'] = SUBDIR_CHARS
    conf['general']['subdir_chars'] = set(str(conf['general']['subdir_chars']))

    if conf['general']['subdir_levels'] > 10:
        conf['general']['subdir_levels'] = 10
        LOG.warning("option subdir_levels must not be greather than 10")

    if not conf['general'].has_key('auth_basic'):
        conf['general']['auth_basic'] = None

    if conf['general'].has_key('web_directories'):
        if isinstance(conf['general']['web_directories'], basestring):
            conf['general']['web_directories'] = [conf['general']['web_directories']]
        elif not isinstance(conf['general']['web_directories'], list):
            LOG.error('Invalid %s type. (%s: %r, section: %r)',
                      'web_directories',
                      'web_directories',
                      conf['general']['web_directories'],
                      'general')
            conf['general']['web_directories'] = []
    else:
        conf['general']['web_directories'] = []

    return conf

def load_conf(xfile, options = None, parse_conf_func = None):
    signal.signal(signal.SIGTERM, stop)
    signal.signal(signal.SIGINT, stop)

    with open(xfile, 'r') as f:
        conf = yaml.load(f)

    if parse_conf_func:
        conf = parse_conf_func(conf)
    else:
        conf = parse_conf(conf)

    for name, module in MODULES.iteritems():
        LOG.info("module init: %r", name)
        module.init(conf)

    for name, plugin in PLUGINS.iteritems():
        if not plugin.enabled:
            continue

        LOG.info("plugin init: %r", name)
        plugin.init(conf)
        LOG.info("plugin safe_init: %r", name)
        plugin.safe_init()
        DWHO_THREADS.append(plugin.at_stop)

    if not options or not isinstance(options, object):
        return conf

    if not getattr(options, 'listen_addr', None):
        setattr(options, 'listen_addr', conf['general']['listen_addr'])

    if not getattr(options, 'listen_port', None):
        setattr(options, 'listen_port', conf['general']['listen_port'])

    setattr(options, 'auth_basic', conf['general']['auth_basic'])
    setattr(options, 'auth_basic_file', conf['general']['auth_basic_file'])
    setattr(options, 'max_body_size', conf['general']['max_body_size'])
    setattr(options, 'max_workers', conf['general']['max_workers'])

    setattr(options, 'configuration', conf)

    return options

def start_plugins():
    for name, plugin in PLUGINS.iteritems():
        if plugin.enabled and plugin.autostart:
            LOG.info("plugin at_start: %r", name)
            plugin.at_start()
